﻿using SYBDContracts.BindingModels;
using SYBDContracts.BusinessLogicsContracts;
using SYBDContracts.SearchModels;
using SYBDContracts.StoragesContracts;
using SYBDContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace SYBDBusinessLogic.BusinessLogics
{
    public class AutoLogic : IAutoLogic
    {
        private readonly ILogger _logger;

        private readonly IAutoStorage _autoStorage;

        public AutoLogic(ILogger<AutoLogic> logger, IAutoStorage autoStorage)
        {
            _logger = logger;
            _autoStorage = autoStorage;
        }

        public List<AutoViewModel>? ReadList(AutoSearchModel? model)
        {
            _logger.LogInformation("ReadList. StateNumber:{StateNumber}. Id:{Id}", model?.StateNumber, model?.Id);
            var list = model == null ? _autoStorage.GetFullList() : _autoStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public AutoViewModel? ReadElement(AutoSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. StateNumber:{StateNumber}. Id:{Id}", model.StateNumber, model.Id);
            var element = _autoStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool Create(AutoBindingModel model)
        {
            CheckModel(model);
            if (_autoStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Update(AutoBindingModel model)
        {
            CheckModel(model);
            if (_autoStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(AutoBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_autoStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(AutoBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Model))
            {
                throw new ArgumentNullException("Нет модели", nameof(model.Model));
            }
            if (string.IsNullOrEmpty(model.StateNumber))
            {
                throw new ArgumentNullException("Нет номеров", nameof(model.StateNumber));
            }
            _logger.LogInformation("Auto. Model:{Model}. StateNumber:{StateNumber}. Id:{Id}", model.Model, model.StateNumber, model.Id);
            var element = _autoStorage.GetElement(new AutoSearchModel
            {
                StateNumber = model.StateNumber
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Автомобиль с таким гос номером уже есть");
            }
        }
    }
}