﻿using SYBDContracts.BindingModels;
using SYBDContracts.BusinessLogicsContracts;
using SYBDContracts.SearchModels;
using SYBDContracts.StoragesContracts;
using SYBDContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace SYBDBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;

        private readonly IOrderStorage _orderStorage;

        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{Id}", model.Id);
            var element = _orderStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool Create(OrderBindingModel model)
        {
            CheckModel(model);
            if (_orderStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Update(OrderBindingModel model)
        {
            CheckModel(model);
            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(OrderBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_orderStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        public bool DeliveryOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, "Выдан");
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, "Готов");
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return ChangeStatus(model, "Выполняется");
        }
        public bool ChangeStatus(OrderBindingModel model, string newStatus)
        {
            var viewModel = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
            if (viewModel == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            
            if (!((viewModel.Status == "В обработке") && (newStatus == "Выполняется") || (viewModel.Status == "Выполняется") && (newStatus == "Готов") || ((viewModel.Status == "Готов") && (newStatus == "Выдан"))))
            {
                return false;
            }
            model.Status = newStatus;
            model.WorkerId = viewModel.WorkerId;
            model.AutoId = viewModel.AutoId;
            model.ClientId = viewModel.ClientId;
            model.InsuranceId = viewModel.InsuranceId;
            model.Price = viewModel.Price;
            model.Description = viewModel.Description;
            CheckModel(model, false);
            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Change status operation failed");
                return false;
            }
            
            return true;
        }
        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Description))
            {
                throw new ArgumentNullException("Нет описания заказа", nameof(model.Description));
            }
            if (model.Price<=0)
            {
                throw new ArgumentNullException("Стоимость должна быть выше 0", nameof(model.Price));
            }
            _logger.LogInformation("Order. Description:{Description}. Price:{Price}.Id:{Id}", model.Description, model.Price, model.Id);
        }
    }
}